/*-----------------------------------------------------------------------------
  name:         nodemcu_humidity
  description:  remote temperature and humidity
  Version:      2016.11.29
  By:           Tim Mikkelsen
  Created:      11/29/2016
  Updated:      12/28/2016

  After doing some SparkFun Thing projects, I wanted to try a
  similar, but less expensive foundation - the NodeMCU.  This is
  a relatively simple remote temperature and humidity sensor
  project.

  Hardware:
  - HiLetgo New Version NodeMCU LUA WiFi Internet ESP8266
  - DHT11 temperature and humidity sensor
  - USB power supply

  Additional libraries and code segments:
  - base wifi server code (by dlf modified by Tom Igoe)
  - Adafruit sensor and DHT libraries
  - example DHT code from Rui Santos

  To make the device work outside of your local network, enable
  your router to port forwarding using your external WAN address with
  the ThingPort value.  Setting the router varies by brand and will
  require looking at the router information.  Once set, an example is:
  - for a   local address   of   192.168.0.98
  - for a   NodeMCU         of   40
  - for a   WAN address     of   50.170.100.101
  -----------------------------------------------------------
  - you would read status   at   http://50.170.100.101:40

*/


// ----------------------------------------------------------------------------
// libraries, definitions, devices, constants, variables
// ----------------------------------------------------------------------------

// libraries  -----------------------------------------------------------------
#include <ESP8266WiFi.h>
#include <DHT.h>

// device constants  ----------------------------------------------------------
#define DHTPin  D1                      // DHT pin - GPIO5 = D1
#define LedRedPin D0                    // LED pin - GPIO16 = D0
#define LedGreenPin D2                  // LED pin - GPIO16 = D2
#define DHTTYPE DHT11                   // DHT 11 sensor

// code constants  ------------------------------------------------------------
const String codeVersion = "Version 2016.12.28";

// variables  -----------------------------------------------------------------
static char celsiusTmpStr[7];
static char fahrenheitTmpStr[7];
static char humidityStr[7];
float h, t, f, hic, hif;

// WiFi and internet variables and constants ----------------------------------
char ssid[] = "Mikkelsen";              // your network SSID (name)
char pass[] = "9702260292";             // your network password
IPAddress myIP(192, 168, 1, 98);        // your desired IP address
IPAddress gateway(192, 168, 1, 1);      // your network gateway address
IPAddress subnet(255, 255, 255, 0);     // subnet mask
int WifiStatus = WL_IDLE_STATUS;
int const IoTPort = 8082;               // port number to allow remote access
// check http://mobinnet.ir/uploaded/common_ports.pdf for unused port

// create devices  ------------------------------------------------------------
WiFiServer server(IoTPort);
DHT dht(DHTPin, DHTTYPE);

// ----------------------------------------------------------------------------
// Arduino setup  -------------------------------------------------------------
// ----------------------------------------------------------------------------
void setup() {

  // initialize serial and wait for port to open:  ----------------------------
  // Serial.begin(9600);                // for debug change '// Serial.' with 'Serial.'

  // initialize I/O  ----------------------------------------------------------
  pinMode(LedRedPin, OUTPUT);
  digitalWrite(LedRedPin, LOW);                             // turn LED off
  pinMode(LedGreenPin, OUTPUT);
  digitalWrite(LedGreenPin, LOW);                           // turn LED off

  // initialize I/O  ----------------------------------------------------------
  dht.begin();                                              // start sensor library

  // use for wifi debug output on serial monitor
  // Serial.setDebugOutput(true);

  // check for the presence of WiFi  ------------------------------------------
  if (WiFi.status() == WL_NO_SHIELD) {
    // Serial.println("WIFI: Hardware not present");
    // don't continue:
    while (true);
  }

  WiFi.mode(WIFI_STA);                                      // make sure Wifi is not an access point

  // attempt to connect to Wifi network:                    // only try if not yet connected
  if ( WiFi.status() != WL_CONNECTED) {
    // Serial.print("WIFI Attempting to connect to SSID: ");
    // Serial.println(ssid);
    WiFi.config(myIP, gateway, subnet);                     // Connect to WPA/WPA2 network.
    WifiStatus = WiFi.begin(ssid, pass);
    // Serial.print("WIFI Status: ");
    // Serial.println(WifiStatus);
    delay(10000);                                           // wait 10 seconds for connection
  }

  server.begin();                                           // start web server

  // blink light to indicate startup
  digitalWrite(LedRedPin, HIGH);                            // turn red LED on
  delay(200);
  digitalWrite(LedGreenPin, HIGH);                          // turn green LED on
  delay(200);
  digitalWrite(LedRedPin, LOW);                             // turn red LED off
  delay(200);
  digitalWrite(LedGreenPin, LOW);                           // turn green LED off
  delay(200);
}



// ----------------------------------------------------------------------------
// Arduino main loop  ---------------------------------------------------------
// ----------------------------------------------------------------------------
void loop() {

  // Serial.print("loop");
  // Check on WiFi staus
  if ( WiFi.status() == WL_CONNECTED) {
    digitalWrite(LedRedPin, HIGH);                          // turn red LED on
    // Serial.print(" WiFi ");
  }
  else {
    digitalWrite(LedRedPin, LOW);                           // turn LED off
    // Serial.print(" no WiFi ");
  }
  // Serial.println(" ");

  WiFiClient client = server.available();                   // listen for incoming clients
  if (client) {
    digitalWrite(LedGreenPin, HIGH);                        // turn LED on to show client action

    // Serial.println("WIFI new client");
    String WebRequest = client.readStringUntil('\r');       // Read the request
    // Serial.println("WIFI Request is '" + WebRequest + "'");

    // send a standard http response header  -----------------------------------
    client.println("HTTP/1.1 200 OK");
    client.println("Content-Type: text/html");
    client.println();
    client.println("<HTML>");
    client.println("<HEAD>");
    client.println("<meta name='viewport' content='width=device-width, initial-scale=1'>");
    client.println("<body bgcolor=AntiqueWhite>");
    client.println("<TITLE>Remote Temperature and Humidity</TITLE>");
    client.println("</HEAD>");
    client.println("<H3>Remote Temperature and Humidity</H3>");
    client.println("<hr width=300 align=LEFT>");
    client.println("<html>");

    // ---------------------------------------------------------------------------
    // web request handling   ----------------------------------------------------
    // ---------------------------------------------------------------------------

    h = dht.readHumidity();                                 // Read humidity
    t = dht.readTemperature();                              // Read temperature as Celsius
    f = dht.readTemperature(true);                          // Read temperature as Fahrenheit
    if (isnan(h) || isnan(t) || isnan(f)) {                 // Check if any reads failed and try again.
      // Serial.println("Failed to read from DHT sensor");
      strcpy(celsiusTmpStr, "Failed");
      strcpy(fahrenheitTmpStr, "Failed");
      strcpy(humidityStr, "Failed");
    }
    else {
      // Computes temperature values in Celsius + Fahrenheit and Humidity
      hic = dht.computeHeatIndex(t, h, false);
      dtostrf(hic, 6, 0, celsiusTmpStr);
      hif = dht.computeHeatIndex(f, h);
      dtostrf(hif, 6, 0, fahrenheitTmpStr);
      dtostrf(h, 6, 0, humidityStr);
    }

    // your actual web page that displays temperature and humidity
    client.println("<!DOCTYPE HTML>");
    client.println("<html>");

    client.print("<progress value='");    
    client.print(f);
    client.print("' max='100'></progress>");
    client.print("  Temperature: ");
    client.println(fahrenheitTmpStr);
    client.println("&deg;F <br />");
    
    client.print("<progress value='");
    client.print(h);
    client.print("' max='100'></progress>");
    client.print("  Humidity:    ");
    client.print(humidityStr);
    client.println("% <br />");

    // ---------------------------------------------------------------------------
    // finish up web page  -------------------------------------------------------
    // ---------------------------------------------------------------------------

    client.println("<hr width=300 align=LEFT>");
    client.println(codeVersion);
    client.println("<br />");
    client.println("<hr width=300 align=LEFT>");
    client.println("</BODY>");
    client.println("</HTML>");
    delay(100);                                             // give browser time to receive data
    client.flush();                                         // close the connection
    client.stop();
    // Serial.println("WIFI client disonnected");
  }

  else {
    // Serial.println("WIFI no request found");
  }
  digitalWrite(LedGreenPin, LOW);                           // turn green LED off

}

